@@ -46,6 +46,8 @@ typedef struct {
         int signal_args[MAX_EXPANDO_SIGNALS];
 } EXPANDO_REC;
 
+const char *current_expando = NULL;
+
 static int timer_tag;
 
 static EXPANDO_REC *char_expandos[255];
@@ -318,6 +320,17 @@ static char *expando_cmdchars(SERVER_REC *server, void *item, int *free_ret)
 	return (char *) settings_get_str("cmdchars");
 }
 
+/* first CMDCHAR */
+static char *expando_cmdchar(SERVER_REC *server, void *item, int *free_ret)
+{
+	char str[2] = { 0, 0 };
+
+	str[0] = *settings_get_str("cmdchars");
+
+	*free_ret = TRUE;
+	return g_strdup(str);
+}
+
 /* modes of current channel, if any */
 static char *expando_chanmode(SERVER_REC *server, void *item, int *free_ret)
 {
@@ -358,7 +371,8 @@ static char *expando_serverversion(SERVER_REC *server, void *item, int *free_ret
 /* target of current input (channel or QUERY nickname) */
 static char *expando_target(SERVER_REC *server, void *item, int *free_ret)
 {
-	return item == NULL ? "" : ((WI_ITEM_REC *) item)->name;
+	return item == NULL ? "" :
+		(char *) window_item_get_target((WI_ITEM_REC *) item);
 }
 
 /* client release date (in YYYYMMDD format) */
@@ -461,6 +475,12 @@ static char *expando_chatnet(SERVER_REC *server, void *item, int *free_ret)
 	return server == NULL ? "" : server->connrec->chatnet;
 }
 
+/* visible_name of current window item */
+static char *expando_itemname(SERVER_REC *server, void *item, int *free_ret)
+{
+	return item == NULL ? "" : ((WI_ITEM_REC *) item)->visible_name;
+}
+
 static void sig_message_public(SERVER_REC *server, const char *msg,
 			       const char *nick, const char *address,
 			       const char *target)
@@ -578,12 +598,15 @@ void expandos_init(void)
 		       "", EXPANDO_NEVER, NULL);
 	expando_create("K", expando_cmdchars,
 		       "setup changed", EXPANDO_ARG_NONE, NULL);
+	expando_create("k", expando_cmdchar,
+		       "setup changed", EXPANDO_ARG_NONE, NULL);
 	expando_create("M", expando_chanmode,
 		       "window changed", EXPANDO_ARG_NONE,
 		       "window item changed", EXPANDO_ARG_WINDOW,
 		       "channel mode changed", EXPANDO_ARG_WINDOW_ITEM, NULL);
 	expando_create("N", expando_nick,
 		       "window changed", EXPANDO_ARG_NONE,
+		       "window connect changed", EXPANDO_ARG_WINDOW,
 		       "window server changed", EXPANDO_ARG_WINDOW,
                        "server nick changed", EXPANDO_ARG_SERVER, NULL);
 	expando_create("O", expando_statusoper,
@@ -611,6 +634,7 @@ void expandos_init(void)
 	expando_create("W", expando_workdir, NULL);
 	expando_create("Y", expando_realname,
 		       "window changed", EXPANDO_ARG_NONE,
+		       "window connect changed", EXPANDO_ARG_WINDOW,
 		       "window server changed", EXPANDO_ARG_WINDOW, NULL);
 	expando_create("Z", expando_time,
 		       "time changed", EXPANDO_ARG_NONE, NULL);
@@ -630,10 +654,17 @@ void expandos_init(void)
 		       "query address changed", EXPANDO_ARG_WINDOW_ITEM, NULL);
 	expando_create("tag", expando_servertag,
 		       "window changed", EXPANDO_ARG_NONE,
+		       "window connect changed", EXPANDO_ARG_WINDOW,
 		       "window server changed", EXPANDO_ARG_WINDOW, NULL);
 	expando_create("chatnet", expando_chatnet,
 		       "window changed", EXPANDO_ARG_NONE,
+		       "window connect changed", EXPANDO_ARG_WINDOW,
 		       "window server changed", EXPANDO_ARG_WINDOW, NULL);
+	expando_create("itemname", expando_itemname,
+		       "window changed", EXPANDO_ARG_NONE,
+		       "window item changed", EXPANDO_ARG_WINDOW,
+		       "window item name changed", EXPANDO_ARG_WINDOW_ITEM,
+		       NULL);
 
 	read_settings();
 
