@@ -2,24 +2,23 @@
 
   client_channel.c
 
-  Author: Pekka Riikonen <priikone@poseidon.pspt.fi>
+  Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1997 - 2001 Pekka Riikonen
+  Copyright (C) 1997 - 2004 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-  
+  the Free Software Foundation; version 2 of the License.
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
 */
-/* $Id: client_channel.c,v 1.41 2002/11/06 13:42:26 priikone Exp $ */
+/* $Id: client_channel.c,v 1.43.2.7 2004/02/28 14:43:29 priikone Exp $ */
 /* This file includes channel message sending and receiving routines,
-   channel key receiving and setting, and channel private key handling 
+   channel key receiving and setting, and channel private key handling
    routines. */
 
 #include "silcincludes.h"
@@ -27,22 +26,21 @@
 #include "client_internal.h"
 
 /* Sends packet to the `channel'. Packet to channel is always encrypted
-   differently from "normal" packets. SILC header of the packet is 
+   differently from "normal" packets. SILC header of the packet is
    encrypted with the next receiver's key and the rest of the packet is
    encrypted with the channel specific key. Padding and HMAC is computed
    with the next receiver's key. The `data' is the channel message. If
    the `force_send' is TRUE then the packet is sent immediately. */
 
-void silc_client_send_channel_message(SilcClient client, 
+bool silc_client_send_channel_message(SilcClient client,
 				      SilcClientConnection conn,
 				      SilcChannelEntry channel,
 				      SilcChannelPrivateKey key,
 				      SilcMessageFlags flags,
-				      unsigned char *data, 
-				      SilcUInt32 data_len, 
-				      int force_send)
+				      unsigned char *data,
+				      SilcUInt32 data_len,
+				      bool force_send)
 {
-  int i;
   SilcSocketConnection sock;
   SilcBuffer payload;
   SilcPacketContext packetdata;
@@ -50,9 +48,9 @@ void silc_client_send_channel_message(SilcClient client,
   SilcCipher cipher;
   SilcHmac hmac;
   unsigned char *id_string;
-  SilcUInt32 iv_len;
   int block_len;
   SilcChannelUser chu;
+  bool ret = FALSE;
 
   assert(client && conn && channel);
   sock = conn->sock;
@@ -61,37 +59,39 @@ void silc_client_send_channel_message(SilcClient client,
   chu = silc_client_on_channel(channel, conn->local_entry);
   if (!chu) {
     SILC_LOG_ERROR(("Cannot send message to channel we are not joined"));
-    return;
+    return FALSE;
   }
 
   /* Check if it is allowed to send messages to this channel by us. */
   if (channel->mode & SILC_CHANNEL_MODE_SILENCE_USERS && !chu->mode)
-    return;
-  if (channel->mode & SILC_CHANNEL_MODE_SILENCE_OPERS && 
+    return FALSE;
+  if (channel->mode & SILC_CHANNEL_MODE_SILENCE_OPERS &&
       chu->mode & SILC_CHANNEL_UMODE_CHANOP &&
       !(chu->mode & SILC_CHANNEL_UMODE_CHANFO))
-    return;
+    return FALSE;
   if (chu->mode & SILC_CHANNEL_UMODE_QUIET)
-    return;
+    return FALSE;
 
   /* Take the key to be used */
-  if (channel->mode & SILC_CHANNEL_MODE_PRIVKEY) {
+  if (channel->private_keys) {
     if (key) {
       /* Use key application specified */
       cipher = key->cipher;
       hmac = key->hmac;
-    } else if (channel->curr_key) {
+    } else if (channel->mode & SILC_CHANNEL_MODE_PRIVKEY &&
+	       channel->curr_key) {
       /* Use current private key */
       cipher = channel->curr_key->cipher;
       hmac = channel->curr_key->hmac;
-    } else if (!channel->curr_key && channel->private_keys) {
-      /* Use just some private key since we don't know what to use 
+    } else if (channel->mode & SILC_CHANNEL_MODE_PRIVKEY &&
+	       !channel->curr_key && channel->private_keys) {
+      /* Use just some private key since we don't know what to use
 	 and private keys are set. */
       silc_dlist_start(channel->private_keys);
       key = silc_dlist_get(channel->private_keys);
       cipher = key->cipher;
       hmac = key->hmac;
-      
+
       /* Use this key as current private key */
       channel->curr_key = key;
     } else {
@@ -105,23 +105,21 @@ void silc_client_send_channel_message(SilcClient client,
     hmac = channel->hmac;
   }
 
-  if (!cipher || !hmac)
-    return;
+  if (!cipher || !hmac) {
+    SILC_LOG_ERROR(("No cipher and HMAC for channel"));
+    return FALSE;
+  }
 
   block_len = silc_cipher_get_block_len(cipher);
 
-  /* Generate IV */
-  iv_len = silc_cipher_get_block_len(cipher);
-  if (channel->iv[0] == '\0')
-    for (i = 0; i < iv_len; i++) channel->iv[i] = 
-				   silc_rng_get_byte(client->rng);
-  else
-    silc_hash_make(client->md5hash, channel->iv, iv_len, channel->iv);
-
-  /* Encode the channel payload. This also encrypts the message payload. */
-  payload = silc_channel_message_payload_encode(flags, data_len, data, iv_len, 
-						channel->iv, cipher, hmac,
-						client->rng);
+  /* Encode the message payload. This also encrypts the message payload. */
+  payload = silc_message_payload_encode(flags, data, data_len, TRUE, FALSE,
+					cipher, hmac, client->rng, NULL,
+					client->private_key, client->sha1hash);
+  if (!payload) {
+    SILC_LOG_ERROR(("Error encoding channel message"));
+    return FALSE;
+  }
 
   /* Get data used in packet header encryption, keys and stuff. */
   cipher = conn->internal->send_key;
@@ -144,11 +142,11 @@ void silc_client_send_channel_message(SilcClient client,
   data_len = SILC_PACKET_DATALEN(data_len, SILC_PACKET_HEADER_LEN +
 				 packetdata.src_id_len +
 				 packetdata.dst_id_len);
-  packetdata.truelen = data_len + SILC_PACKET_HEADER_LEN + 
+  packetdata.truelen = data_len + SILC_PACKET_HEADER_LEN +
     packetdata.src_id_len + packetdata.dst_id_len;
-  packetdata.padlen = SILC_PACKET_PADLEN((SILC_PACKET_HEADER_LEN +
-					  packetdata.src_id_len +
-					  packetdata.dst_id_len), block_len);
+  SILC_PACKET_PADLEN((SILC_PACKET_HEADER_LEN +
+		      packetdata.src_id_len +
+		      packetdata.dst_id_len), block_len, packetdata.padlen);
 
   /* Create the outgoing packet */
   if (!silc_packet_assemble(&packetdata, client->rng, cipher, hmac, sock,
@@ -157,10 +155,10 @@ void silc_client_send_channel_message(SilcClient client,
     goto out;
   }
 
-  /* Encrypt the header and padding of the packet. This is encrypted 
+  /* Encrypt the header and padding of the packet. This is encrypted
      with normal session key shared with our server. */
   silc_packet_encrypt(cipher, hmac, conn->internal->psn_send++,
-		      (SilcBuffer)&packet, SILC_PACKET_HEADER_LEN + 
+		      (SilcBuffer)&packet, SILC_PACKET_HEADER_LEN +
 		      packetdata.src_id_len + packetdata.dst_id_len +
 		      packetdata.padlen);
 
@@ -170,14 +168,25 @@ void silc_client_send_channel_message(SilcClient client,
   /* Now actually send the packet */
   silc_client_packet_send_real(client, sock, force_send);
 
+  /* Check for mandatory rekey */
+  if (conn->internal->psn_send == SILC_CLIENT_REKEY_THRESHOLD)
+    silc_schedule_task_add(client->schedule, sock->sock,
+			   silc_client_rekey_callback, sock, 0, 1,
+			   SILC_TASK_TIMEOUT, SILC_TASK_PRI_NORMAL);
+
+  ret = TRUE;
+
  out:
   silc_buffer_free(payload);
   silc_free(id_string);
+
+  return ret;
 }
 
 typedef struct {
-  SilcChannelMessagePayload payload;
+  SilcMessagePayload payload;
   SilcChannelID *channel_id;
+  SilcChannelPrivateKey key;
 } *SilcChannelClientResolve;
 
 static void silc_client_channel_message_cb(SilcClient client,
@@ -207,40 +216,41 @@ static void silc_client_channel_message_cb(SilcClient client,
       silc_hash_table_add(clients[0]->channels, channel, chu);
     }
 
-    message = silc_channel_message_get_data(res->payload, &message_len);
-    
+    message = silc_message_get_data(res->payload, &message_len);
+
     /* Pass the message to application */
     client->internal->ops->channel_message(
-			       client, conn, clients[0], channel,
-			       silc_channel_message_get_flags(res->payload),
-			       message, message_len);
+			    client, conn, clients[0], channel, res->payload,
+			    res->key, silc_message_get_flags(res->payload),
+			    message, message_len);
   }
 
  out:
-  silc_channel_message_payload_free(res->payload);
+  silc_message_payload_free(res->payload);
   silc_free(res->channel_id);
   silc_free(res);
 }
 
 /* Process received message to a channel (or from a channel, really). This
    decrypts the channel message with channel specific key and parses the
-   channel payload. Finally it displays the message on the screen. */
+   message payload. Finally it displays the message on the screen. */
 
-void silc_client_channel_message(SilcClient client, 
-				 SilcSocketConnection sock, 
+void silc_client_channel_message(SilcClient client,
+				 SilcSocketConnection sock,
 				 SilcPacketContext *packet)
 {
   SilcClientConnection conn = (SilcClientConnection)sock->user_data;
   SilcBuffer buffer = packet->buffer;
-  SilcChannelMessagePayload payload = NULL;
+  SilcMessagePayload payload = NULL;
   SilcChannelID *id = NULL;
   SilcChannelEntry channel;
   SilcClientEntry client_entry;
   SilcClientID *client_id = NULL;
   unsigned char *message;
   SilcUInt32 message_len;
+  SilcChannelPrivateKey key = NULL;
 
-  SILC_LOG_DEBUG(("Start"));
+  SILC_LOG_DEBUG(("Received channel message"));
 
   /* Sanity checks */
   if (packet->dst_id_type != SILC_ID_CHANNEL)
@@ -259,46 +269,67 @@ void silc_client_channel_message(SilcClient client,
   if (!channel)
     goto out;
 
-  /* If there is no channel private key then just decrypt the message 
+  /* If there is no channel private key then just decrypt the message
      with the channel key. If private keys are set then just go through
      all private keys and check what decrypts correctly. */
-  if (!(channel->mode & SILC_CHANNEL_MODE_PRIVKEY)) {
+  if (!channel->private_keys) {
     /* Parse the channel message payload. This also decrypts the payload */
-    payload = silc_channel_message_payload_parse(buffer->data, buffer->len, 
-						 channel->channel_key,
-						 channel->hmac);
+    payload = silc_message_payload_parse(buffer->data, buffer->len, FALSE,
+					 FALSE, channel->channel_key,
+					 channel->hmac);
 
     /* If decryption failed and we have just performed channel key rekey
        we will use the old key in decryption. If that fails too then we
        cannot do more and will drop the packet. */
     if (!payload) {
-      if (!channel->old_channel_key) {
-	goto out;
-      }
+      SilcCipher key;
+      SilcHmac hmac;
+      int i;
 
-      payload = silc_channel_message_payload_parse(buffer->data, buffer->len, 
-						   channel->old_channel_key,
-						   channel->old_hmac);
-      if (!payload) {
+      if (!channel->old_channel_keys ||
+	  !silc_dlist_count(channel->old_channel_keys))
 	goto out;
-      }
-    }
-  } else if (channel->private_keys) {
-    SilcChannelPrivateKey entry;
 
-    silc_dlist_start(channel->private_keys);
-    while ((entry = silc_dlist_get(channel->private_keys)) != SILC_LIST_END) {
-      /* Parse the channel message payload. This also decrypts the payload */
-      payload = silc_channel_message_payload_parse(buffer->data, buffer->len, 
-						   entry->cipher,
-						   entry->hmac);
-      if (payload)
-	break;
+      SILC_LOG_DEBUG(("Attempting to decrypt with old channel key(s)"));
+
+      silc_dlist_end(channel->old_channel_keys);
+      silc_dlist_end(channel->old_hmacs);
+      for (i = 0; i < silc_dlist_count(channel->old_channel_keys); i++) {
+	key = silc_dlist_get(channel->old_channel_keys);
+	hmac = silc_dlist_get(channel->old_hmacs);
+	if (!key || !hmac)
+	  break;
+
+	payload = silc_message_payload_parse(buffer->data, buffer->len,
+					     FALSE, FALSE, key, hmac);
+	if (payload)
+	  break;
+      }
+      if (!payload)
+	goto out;
     }
-    if (entry == SILC_LIST_END)
-      goto out;
   } else {
-    goto out;
+    /* If the private key mode, however is not set on the channel then
+       try the actual channel key first before trying private keys. */
+    if (!(channel->mode & SILC_CHANNEL_MODE_PRIVKEY))
+      payload = silc_message_payload_parse(buffer->data, buffer->len, FALSE,
+					   FALSE, channel->channel_key,
+					   channel->hmac);
+
+    if (!payload) {
+      silc_dlist_start(channel->private_keys);
+      while ((key = silc_dlist_get(channel->private_keys))
+	     != SILC_LIST_END) {
+	/* Parse the message payload. This also decrypts the payload */
+	payload = silc_message_payload_parse(buffer->data, buffer->len,
+					     FALSE, FALSE,
+					     key->cipher, key->hmac);
+	if (payload)
+	  break;
+      }
+      if (key == SILC_LIST_END)
+	goto out;
+    }
   }
 
   /* Find client entry */
@@ -309,6 +340,7 @@ void silc_client_channel_message(SilcClient client,
     SilcChannelClientResolve res = silc_calloc(1, sizeof(*res));
     res->payload = payload;
     res->channel_id = id;
+    res->key = key;
     silc_client_get_client_by_id_resolve(client, conn, client_id, NULL,
 					 silc_client_channel_message_cb,
 					 res);
@@ -317,45 +349,57 @@ void silc_client_channel_message(SilcClient client,
     goto out;
   }
 
-  message = silc_channel_message_get_data(payload, &message_len);
+  message = silc_message_get_data(payload, &message_len);
 
   /* Pass the message to application */
   client->internal->ops->channel_message(
-				 client, conn, client_entry, channel,
-				 silc_channel_message_get_flags(payload),
-				 message, message_len);
+			     client, conn, client_entry, channel, payload,
+			     key, silc_message_get_flags(payload),
+			     message, message_len);
 
  out:
   silc_free(id);
   silc_free(client_id);
   if (payload)
-    silc_channel_message_payload_free(payload);
+    silc_message_payload_free(payload);
 }
 
 /* Timeout callback that is called after a short period of time after the
-   new channel key has been created. This removes the old channel key all
-   together. */
+   new channel key has been created.  This removes the first channel key
+   in the list. */
 
 SILC_TASK_CALLBACK(silc_client_save_channel_key_rekey)
 {
   SilcChannelEntry channel = (SilcChannelEntry)context;
+  SilcCipher key;
+  SilcHmac hmac;
 
-  if (channel->old_channel_key)
-    silc_cipher_free(channel->old_channel_key);
-  if (channel->old_hmac)
-    silc_hmac_free(channel->old_hmac);
-  channel->old_channel_key = NULL;
-  channel->old_hmac = NULL;
-  channel->rekey_task = NULL;
+  if (channel->old_channel_keys) {
+    silc_dlist_start(channel->old_channel_keys);
+    key = silc_dlist_get(channel->old_channel_keys);
+    if (key) {
+      silc_dlist_del(channel->old_channel_keys, key);
+      silc_cipher_free(key);
+    }
+  }
+
+  if (channel->old_hmacs) {
+    silc_dlist_start(channel->old_hmacs);
+    hmac = silc_dlist_get(channel->old_hmacs);
+    if (hmac) {
+      silc_dlist_del(channel->old_hmacs, hmac);
+      silc_hmac_free(hmac);
+    }
+  }
 }
 
 /* Saves channel key from encoded `key_payload'. This is used when we
-   receive Channel Key Payload and when we are processing JOIN command 
+   receive Channel Key Payload and when we are processing JOIN command
    reply. */
 
 void silc_client_save_channel_key(SilcClient client,
 				  SilcClientConnection conn,
-				  SilcBuffer key_payload, 
+				  SilcBuffer key_payload,
 				  SilcChannelEntry channel)
 {
   unsigned char *id_string, *key, *cipher, *hmac, hash[32];
@@ -387,24 +431,21 @@ void silc_client_save_channel_key(SilcClient client,
       goto out;
   }
 
-  hmac = (channel->hmac ? (char *)silc_hmac_get_name(channel->hmac) : 
+  hmac = (channel->hmac ? (char *)silc_hmac_get_name(channel->hmac) :
 	  SILC_DEFAULT_HMAC);
 
   /* Save the old key for a short period of time so that we can decrypt
      channel message even after the rekey if some client would be sending
      messages with the old key after the rekey. */
-  if (channel->old_channel_key)
-    silc_cipher_free(channel->old_channel_key);
-  if (channel->old_hmac)
-    silc_hmac_free(channel->old_hmac);
-  if (channel->rekey_task)
-    silc_schedule_task_del(client->schedule, channel->rekey_task);
-  channel->old_channel_key = channel->channel_key;
-  channel->old_hmac = channel->hmac;
-  channel->rekey_task = 
-    silc_schedule_task_add(client->schedule, 0,
-			   silc_client_save_channel_key_rekey, channel,
-			   10, 0, SILC_TASK_TIMEOUT, SILC_TASK_PRI_NORMAL);
+  if (!channel->old_channel_keys)
+    channel->old_channel_keys = silc_dlist_init();
+  if (!channel->old_hmacs)
+    channel->old_hmacs = silc_dlist_init();
+  silc_dlist_add(channel->old_channel_keys, channel->channel_key);
+  silc_dlist_add(channel->old_hmacs, channel->hmac);
+  silc_schedule_task_add(client->schedule, 0,
+			 silc_client_save_channel_key_rekey, channel,
+			 10, 0, SILC_TASK_TIMEOUT, SILC_TASK_PRI_NORMAL);
 
   /* Free the old channel key data */
   silc_free(channel->key);
@@ -417,9 +458,9 @@ void silc_client_save_channel_key(SilcClient client,
 
   if (!silc_cipher_alloc(cipher, &channel->channel_key)) {
     client->internal->ops->say(
-			   conn->client, conn, 
+			   conn->client, conn,
 			   SILC_CLIENT_MESSAGE_AUDIT,
-			   "Cannot talk to channel: unsupported cipher %s", 
+			   "Cannot talk to channel: unsupported cipher %s",
 			   cipher);
     goto out;
   }
@@ -430,7 +471,7 @@ void silc_client_save_channel_key(SilcClient client,
   /* Generate HMAC key from the channel key data and set it */
   silc_hmac_alloc(hmac, NULL, &channel->hmac);
   silc_hash_make(silc_hmac_get_hash(channel->hmac), key, tmp_len, hash);
-  silc_hmac_set_key(channel->hmac, hash, 
+  silc_hmac_set_key(channel->hmac, hash,
 		    silc_hash_len(silc_hmac_get_hash(channel->hmac)));
   memset(hash, 0, sizeof(hash));
 
@@ -441,7 +482,7 @@ void silc_client_save_channel_key(SilcClient client,
 
 /* Processes received key for channel. The received key will be used
    to protect the traffic on the channel for now on. Client must receive
-   the key to the channel before talking on the channel is possible. 
+   the key to the channel before talking on the channel is possible.
    This is the key that server has generated, this is not the channel
    private key, it is entirely local setting. */
 
@@ -455,25 +496,24 @@ void silc_client_receive_channel_key(SilcClient client,
   silc_client_save_channel_key(client, sock->user_data, packet, NULL);
 }
 
-/* Adds private key for channel. This may be set only if the channel's mode
-   mask includes the SILC_CHANNEL_MODE_PRIVKEY. This returns FALSE if the
-   mode is not set. When channel has private key then the messages are
-   encrypted using that key. All clients on the channel must also know the
-   key in order to decrypt the messages. However, it is possible to have
-   several private keys per one channel. In this case only some of the
-   clients on the channel may know the one key and only some the other key.
+/* Adds private key for channel. When channel has private key then the
+   messages are encrypted using that key. All clients on the channel must
+   also know the key in order to decrypt the messages. However, it is
+   possible to have several private keys per one channel. In this case
+   only some of the clients on the channel may know the one key and only
+   some the other key.
 
-   If `cipher' and/or `hmac' is NULL then default values will be used 
+   If `cipher' and/or `hmac' is NULL then default values will be used
    (aes-256-cbc for cipher and hmac-sha1-96 for hmac).
 
    The private key for channel is optional. If it is not set then the
    channel messages are encrypted using the channel key generated by the
-   server. However, setting the private key (or keys) for the channel 
+   server. However, setting the private key (or keys) for the channel
    significantly adds security. If more than one key is set the library
    will automatically try all keys at the message decryption phase. Note:
    setting many keys slows down the decryption phase as all keys has to
    be tried in order to find the correct decryption key. However, setting
-   a few keys does not have big impact to the decryption performace. 
+   a few keys does not have big impact to the decryption performace.
 
    NOTE: that this is entirely local setting. The key set using this function
    is not sent to the network at any phase.
@@ -484,14 +524,15 @@ void silc_client_receive_channel_key(SilcClient client,
    currently it is not expected that the SKE key material would be used
    as channel private key. However, this API allows it. */
 
-int silc_client_add_channel_private_key(SilcClient client,
-					SilcClientConnection conn,
-					SilcChannelEntry channel,
-					const char *name,
-					char *cipher,
-					char *hmac,
-					unsigned char *key,
-					SilcUInt32 key_len)
+bool silc_client_add_channel_private_key(SilcClient client,
+					 SilcClientConnection conn,
+					 SilcChannelEntry channel,
+					 const char *name,
+					 char *cipher,
+					 char *hmac,
+					 unsigned char *key,
+					 SilcUInt32 key_len,
+					 SilcChannelPrivateKey *ret_key)
 {
   SilcChannelPrivateKey entry;
   unsigned char hash[32];
@@ -499,9 +540,6 @@ int silc_client_add_channel_private_key(SilcClient client,
 
   assert(client && channel);
 
-  if (!(channel->mode & SILC_CHANNEL_MODE_PRIVKEY))
-    return FALSE;
-
   if (!cipher)
     cipher = SILC_DEFAULT_CIPHER;
   if (!hmac)
@@ -515,25 +553,11 @@ int silc_client_add_channel_private_key(SilcClient client,
 
   /* Produce the key material */
   keymat = silc_calloc(1, sizeof(*keymat));
-  if (silc_ske_process_key_material_data(key, key_len, 16, 256, 16, 
-					 client->md5hash, keymat) 
+  if (silc_ske_process_key_material_data(key, key_len, 16, 256, 16,
+					 client->sha1hash, keymat)
       != SILC_SKE_STATUS_OK)
     return FALSE;
 
-  /* Remove the current key, if it exists. */
-  if (channel->channel_key) {
-    silc_cipher_free(channel->channel_key);
-    memset(channel->key, 0, channel->key_len / 8);
-    silc_free(channel->key);
-    channel->channel_key = NULL;
-    channel->key = NULL;
-    channel->key_len = 0;
-  }
-  if (channel->hmac) {
-    silc_hmac_free(channel->hmac);
-    channel->hmac = NULL;
-  }
-
   if (!channel->private_keys)
     channel->private_keys = silc_dlist_init();
 
@@ -549,9 +573,9 @@ int silc_client_add_channel_private_key(SilcClient client,
 
   /* Generate HMAC key from the channel key data and set it */
   silc_hmac_alloc(hmac, NULL, &entry->hmac);
-  silc_hash_make(silc_hmac_get_hash(entry->hmac), entry->key, 
+  silc_hash_make(silc_hmac_get_hash(entry->hmac), entry->key,
 		 entry->key_len, hash);
-  silc_hmac_set_key(entry->hmac, hash, 
+  silc_hmac_set_key(entry->hmac, hash,
 		    silc_hash_len(silc_hmac_get_hash(entry->hmac)));
   memset(hash, 0, sizeof(hash));
 
@@ -564,6 +588,9 @@ int silc_client_add_channel_private_key(SilcClient client,
   /* Free the key material */
   silc_ske_free_key_material(keymat);
 
+  if (ret_key)
+    *ret_key = entry;
+
   return TRUE;
 }
 
@@ -571,7 +598,7 @@ int silc_client_add_channel_private_key(SilcClient client,
    after calling this to protect the channel messages. Returns FALSE on
    on error, TRUE otherwise. */
 
-int silc_client_del_channel_private_keys(SilcClient client,
+bool silc_client_del_channel_private_keys(SilcClient client,
 					 SilcClientConnection conn,
 					 SilcChannelEntry channel)
 {
@@ -607,7 +634,7 @@ int silc_client_del_channel_private_keys(SilcClient client,
    old channel key is used hereafter to protect the channel messages. This
    returns FALSE on error, TRUE otherwise. */
 
-int silc_client_del_channel_private_key(SilcClient client,
+bool silc_client_del_channel_private_key(SilcClient client,
 					SilcClientConnection conn,
 					SilcChannelEntry channel,
 					SilcChannelPrivateKey key)
@@ -699,7 +726,7 @@ void silc_client_current_channel_private_key(SilcClient client,
   channel->curr_key = key;
 }
 
-/* Returns the SilcChannelUser entry if the `client_entry' is joined on the 
+/* Returns the SilcChannelUser entry if the `client_entry' is joined on the
    channel indicated by the `channel'. NULL if client is not joined on
    the channel. */
 
@@ -708,7 +735,7 @@ SilcChannelUser silc_client_on_channel(SilcChannelEntry channel,
 {
   SilcChannelUser chu;
 
-  if (silc_hash_table_find(channel->user_list, client_entry, NULL, 
+  if (silc_hash_table_find(channel->user_list, client_entry, NULL,
 			   (void *)&chu))
     return chu;
 
