@@ -22,9 +22,7 @@
 #define CIPHERS_H
 
 #include "none.h"
-#include "mars.h"
 #include "rc5.h"
-#include "rc6.h"
 #include "twofish.h"
 #include "aes.h"
 #include "blowfish.h"
