@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silcbuffmt.c,v 1.16.2.1 2002/12/29 21:27:38 priikone Exp $ */
+/* $Id: silcbuffmt.c,v 1.16.2.3 2004/02/23 16:51:08 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -524,7 +524,7 @@ int silc_buffer_strformat(SilcBuffer dst, ...)
 
   /* Parse the arguments by formatting type. */
   while(1) {
-    char *string = va_arg(va, char *);
+    char *string = (char *)va_arg(va, void *);
 
     if (!string)
       continue;
@@ -532,9 +532,12 @@ int silc_buffer_strformat(SilcBuffer dst, ...)
       goto ok;
 
     dst->head = silc_realloc(dst->head, sizeof(*dst->head) *
-			     (strlen(string) + len));
+			     (strlen(string) + len + 1));
+    if (!dst->head)
+      return -1;
     memcpy(dst->head + len, string, strlen(string));
     len += strlen(string);
+    dst->head[len] = '\0';
   }
 
   SILC_LOG_DEBUG(("Error occured while formatting buffer"));
@@ -543,9 +546,9 @@ int silc_buffer_strformat(SilcBuffer dst, ...)
 
  ok:
   dst->end = dst->head + len;
-  dst->tail = dst->data = dst->end;
-  dst->len = 0;
-  dst->truelen = len;
+  dst->data = dst->head;
+  dst->tail = dst->end;
+  dst->len = dst->truelen = len;
 
   va_end(va);
   return len;
