@@ -4,7 +4,7 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1997 - 2003 Pekka Riikonen
+  Copyright (C) 1997 - 2004 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: command_reply.c,v 1.106.2.6 2003/11/07 18:19:04 priikone Exp $ */
+/* $Id: command_reply.c,v 1.106.2.10 2004/02/24 14:33:17 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -885,8 +885,32 @@ SILC_SERVER_CMD_REPLY_FUNC(motd)
   if (!entry) {
     entry = silc_idlist_find_server_by_id(server->global_list, server_id,
 					  TRUE, NULL);
-    if (!entry)
-      goto out;
+    if (!entry) {
+      SilcBuffer buffer;
+
+      /* If router did not find such Server ID in its lists then this must
+	 be bogus client or some router in the net is buggy. */
+      if (server->server_type != SILC_SERVER)
+	goto out;
+
+      /* entry isn't known so we IDENTIFY it. otherwise the
+         silc_server_command_motd won't know about it and tell
+         the client that there is no such server */
+      buffer = silc_command_payload_encode_va(SILC_COMMAND_IDENTIFY,
+	  				      ++server->cmd_ident, 5,
+					      1, NULL, 0, 2, NULL, 0,
+					      3, NULL, 0, 4, NULL, 0,
+					      5, tmp, tmp_len);
+      silc_server_packet_send(server, SILC_PRIMARY_ROUTE(server),
+	  		      SILC_PACKET_COMMAND, 0, buffer->data,
+			      buffer->len, TRUE);
+      silc_server_command_pending(server, SILC_COMMAND_IDENTIFY,
+	  			  server->cmd_ident,
+				  silc_server_command_reply_motd,
+				  cmd);
+      silc_buffer_free(buffer);
+      return;
+    }
   }
 
   /* Get the motd */
@@ -1122,6 +1146,11 @@ SILC_SERVER_CMD_REPLY_FUNC(join)
   if (tmp && server->server_type == SILC_SERVER)
     silc_server_set_channel_pk_list(server, NULL, entry, tmp, len);
 
+  /* The the user limit */
+  tmp = silc_argument_get_arg_type(cmd->args, 17, &len);
+  if (tmp && len == 4)
+    SILC_GET32_MSB(entry->user_limit, tmp);
+
   /* If channel was not created we know there is global users on the
      channel. */
   entry->global_users = (created == 0 ? TRUE : FALSE);
@@ -1345,12 +1374,14 @@ SILC_SERVER_CMD_REPLY_FUNC(getkey)
 	goto out;
     }
 
-    if (!silc_hash_table_find_by_context(server->pk_hash, public_key,
-					 client, NULL))
-      silc_hash_table_add(server->pk_hash, public_key, client);
+    if (!client->data.public_key) {
+      if (!silc_hash_table_find_by_context(server->pk_hash, public_key,
+					   client, NULL))
+	silc_hash_table_add(server->pk_hash, public_key, client);
 
-    client->data.public_key = public_key;
-    public_key = NULL;
+      client->data.public_key = public_key;
+      public_key = NULL;
+    }
   } else if (id_type == SILC_ID_SERVER) {
     server_id = silc_id_payload_get_id(idp);
 
